/*
Terrain rendering skeleton, mostly calls terrain controller library.

Orion Sky Lawlor, olawlor@acm.org, 2003/10/14
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string>
#include "osl/random.h"
#include "ogl/util.h"
#include "ogl/main.h"
#include "terrain/controller.h"
using namespace osl;
using namespace osl::graphics2d;

class testingTerrainController : public TerrainController
{
	bool forTest;
public:
	testingTerrainController(TerrainSet *t,bool forTest_)
		:TerrainController(t), forTest(forTest_) 
	{
		errTol=1.0; /* increase error tolerance, for better Windows framerate */
	}
	
	virtual void draw(const osl::Viewpoint &vp) {
		if (forTest) timeTests();
		TerrainController::draw(vp);
	}
};

void breakhere(void) {}

int main(int argc,char **argv) {
	oglOptions i("Terrain Renderer");
	i.lighting=false;
	//if (w) {i.width=w; i.height=h;}
	i.depth=true;
	i.depthRange=2.0e2;
	i.clearColor=// osl::Vector3d(0,0,0);
		osl::Vector3d(0.4,0.6,0.8); /* clear blue sky */
	oglSetup(i,&argc,argv);

	std::string baseName="src";
	if (argc>1) baseName=argv[1];
	int w=0,h=0;
	if (argc>2) sscanf(argv[2],"%dx%d",&w,&h);
		
	TerrainSet *t=NULL;
	try {
		double startTime=oglTimer();
		t=new TerrainSet(baseName.c_str());
		if (t->needsWrite) {
			t->write();
		}
		printf("Setup took %.2f s\n",oglTimer()-startTime);
	} catch (Exception *e) {
		e->printStackTrace();
		return 0;
	}
	bool doTest=false;
	if (argc>3) doTest=true;
	testingTerrainController *d=new testingTerrainController(t,doTest);

	glutPostRedisplay();
	oglEventloop(d,d);
	return 0;
}
