/**
Portable Memory-Mapped file interface
Orion Sky Lawlor, olawlor@acm.org, 2003/12/17
*/
#include <string.h>
#include "mmap.h"
#include "osl/io.h"
using namespace osl::io;

#if 0
/* UNIX mmap */
#include <sys/mman.h>
#include <unistd.h>
#include <fcntl.h>

int access2open_perm(MmapAccess access) {
	switch (access) {
	case MmapRead: return O_RDONLY;
	case MmapWrite: return O_CREAT+O_WRONLY;
	case MmapReadWrite: return O_RDWR;
	};
	return 0;
}
int access2mmap_perm(MmapAccess access) {
	switch (access) {
	case MmapRead: return PROT_READ;
	case MmapWrite: return PROT_WRITE;
	case MmapReadWrite: return PROT_READ+PROT_WRITE;
	};
	return 0;
}

osl::io::MmapFile::MmapFile(const File &f_,unsigned long len_,MmapAccess access_)  
	:f(f_), len(len_), access(access_), data(0)
{
	int fd=open(f.toString(),access2open_perm(access),0777);
	if (fd==-1) 
		OSL_THROW(IOException, (String("Error opening file ")+f.toString()+" for mmap").c_str());
	data=mmap(0,len,access2mmap_perm(access), MAP_SHARED,fd,0);
	close(fd);
	if (data==(void *)-1) 
		OSL_THROW(IOException, (String("Error mmaping file ")+f.toString()).c_str());
}
osl::io::MmapFile::~MmapFile() {
	munmap(data,len);
}

#else
/* No mmap implementation available: just use file I/O */
osl::io::MmapFile::MmapFile(const File &f_,unsigned long len_,MmapAccess access_) 
	:f(f_), len(len_), access(access_), data(new char[len])
{
	switch (access) {
	case MmapRead:
	case MmapReadWrite: {
		osl::io::FileInputStream fs(f);
		fs.read(data,len);
		} break;
	case MmapWrite: {
		memset(data,0,len);
		} break;
	};
}
osl::io::MmapFile::~MmapFile() {
	if (access==MmapWrite || access==MmapReadWrite) {
		osl::io::FileOutputStream fs(f);
		fs.write(data,len);
	}
	delete[] (char *)data;
}

#endif
