/**
 Terrain Controller-- flies around a terrain.
 
 Orion Sky Lawlor, olawlor@acm.org, 2004/4/6
*/
#ifndef __TERRAIN_CONTROLLER_H
#define __TERRAIN_CONTROLLER_H

#include "terrain/terrain.h"
#include "ogl/main.h"
#include "osl/vector2d.h"
using osl::Polar2d;
using osl::Vector2d;

/**
 A terrain controller manages user interaction with a terrain.
 
 You use it by:
 	- Creating or reading a TerrainSet.
	- Every frame:
		- Feed the controller events.
		- getViewpoint
		- Convert the viewpoint to an OpenGL matrix
		- Upload the matrix into OpenGL
		- draw the terrain
*/
class TerrainController : public oglDrawable, public oglController {
protected:
	TerrainSet *t;
	
	char msgBuf[200];
	int win_size;
	
	// Camera parameters: (FIXME: make separate controller) 
	osl::Viewpoint c_viewpoint;
	int lastX, lastY;
	enum {stateInvalid, stateRotate, stateTrans} state;
	bool demoMode;
	Vector3d demoTarget;
	
	Vector2d eyeDir2d(void) { /* straight ahead from eye */
		return Polar2d(1.0,eyeHeading);
	}
	Vector2d eyeRight2d(void) { /* right angle from eye */
		return Polar2d(1.0,eyeHeading-0.5*M_PI);
	}
	Vector3d lookDir(void) {
		Vector2d h=cos(eyeDip)*eyeDir2d();
		return Vector3d(h.x,h.y,sin(eyeDip));
	}
	virtual Vector3d upDir(void) {
		return Vector3d(0,0,1);
	}
	
	// Drawing and terrain utils:
	int drawMode;
	void flipBit(int drawMode_flip,const char *what);
	void setHt(double h);
	void setErr(double e);
	
	double errTol;
	
	bool useSaved;
	osl::Viewpoint savedView;
public:
	/// Camera parameters:
	Vector2d eyePoint; ///< Map coordinates of eye
	double eyeHeight; ///< Elevation (meters) of eye
	double eyeHeading; ///< Heading (radians counterclockwise from due east)
	double eyeDip; ///< Vertical view angle, measured up from horizon (radians)
	double eyeFOV; ///< Field of View (radians)
	
	double uiSpeed; ///<- Rate at which user interface affects viewpoint
	
	TerrainController(TerrainSet *t_);
	
	void timeTests(void);
	void timeTest(int wid,int drawMode,int skipDisp);
	
	/// Set the screen-space error tolerance, in pixels.
	void setErrorTolerance(double p) {errTol=p;}
	
	void drawTo(const osl::Viewpoint &v,TerrainRenderer *renderer);
	void draw(const osl::Viewpoint &v);
	
	// oglController interface:
	virtual void handleEvent(int uiCode);
	const osl::Viewpoint &getViewpoint(int width,int height);
	void getZRange(double &z_near,double &z_far);
	void mouseDown(int button,int x,int y);
	void mouseDrag(int buttonMask,int x,int y);
	void mouseUp(int buttonMask,int x,int y);
	
	oglTexture *compass;
}; 

#endif
